----------------------------------------------------------------------
--  Adadep - Main program body                                      --
--  Copyright (C) 2002, 2012 Adalog                                 --
--  Author: J-P. Rosen                                              --
--                                                                  --
--  ADALOG   is   providing   training,   consultancy,   expertise, --
--  assistance and custom developments  in Ada and related software --
--  engineering techniques.  For more info about our services:      --
--  ADALOG                          Tel: +33 1 45 29 21 52          --
--  2 rue du Docteur Lombard        Fax: +33 1 45 29 25 00          --
--  92441 ISSY LES MOULINEAUX CEDEX E-m: info@adalog.fr             --
--  FRANCE                          URL: http://www.adalog.fr       --
--                                                                  --
--  This  unit is  free software;  you can  redistribute  it and/or --
--  modify  it under  terms of  the GNU  General Public  License as --
--  published by the Free Software Foundation; either version 2, or --
--  (at your  option) any later version.  This  unit is distributed --
--  in the hope  that it will be useful,  but WITHOUT ANY WARRANTY; --
--  without even the implied warranty of MERCHANTABILITY or FITNESS --
--  FOR A  PARTICULAR PURPOSE.  See the GNU  General Public License --
--  for more details.   You should have received a  copy of the GNU --
--  General Public License distributed  with this program; see file --
--  COPYING.   If not, write  to the  Free Software  Foundation, 59 --
--  Temple Place - Suite 330, Boston, MA 02111-1307, USA.           --
--                                                                  --
--  As  a special  exception, if  other files  instantiate generics --
--  from  this unit,  or you  link this  unit with  other  files to --
--  produce an executable,  this unit does not by  itself cause the --
--  resulting executable  to be covered  by the GNU  General Public --
--  License.  This exception does  not however invalidate any other --
--  reasons why  the executable  file might be  covered by  the GNU --
--  Public License.                                                 --
----------------------------------------------------------------------

with   -- Standard Ada units
  Ada.Characters.Handling,
  Ada.Exceptions;

with   -- ASIS components
  Asis.Ada_Environments,
  Asis.Errors,
  Asis.Exceptions,
  Asis.Implementation;

with   -- Application specific units
  Adadep_Options,
  Units_List,
  Utilities,
  Analyzer;

procedure Adadep is
   use Utilities;
   use Asis.Implementation, Asis.Exceptions;

   use Ada.Characters.Handling, Adadep_Options, Analyzer;
   My_Context : aliased Asis.Context;

   Recursion_Modes : constant array (Boolean) of Units_List.Recursion_Mode := (False => Units_List.None,
                                                                               True  => Units_List.Full);
begin  -- Adadep

   Analyze_Options;

   --
   -- Initialize environment
   --
   Asis.Implementation.Initialize (Initialize_String);
   Asis.Ada_Environments.Associate (My_Context, "Adadep", Parameters_String);
   Asis.Ada_Environments.Open (My_Context);

   --
   -- Register unit specification
   --
   Units_List.Initialize (My_Context'Unchecked_Access);
   Units_List.Register (Units_String,
                        Recursion  => Recursion_Modes (Recursive_Option),
                        Add_Stubs  => True);

   --
   -- Do the job
   --
   Units_List.Reset;
   while not Units_List.Is_Exhausted loop
      declare
         Unit_Name : constant Wide_String := Units_List.Current_Unit;
      begin
         if Spec_Option then
            User_Log ("Analyzing " & Unit_Name & " (spec)");
            Analyzer.Process(Unit_Name, Specification_Part, My_Context);
            if not Merge_Option then
               Analyzer.Produce_Result (Unit_Name, Specification_Part);
            end if;
         end if;

         if Body_Option then
            Analyzer.Process(Unit_Name, Body_Part, My_Context);
            User_Log ("Analyzing " & Unit_Name & " (body)");
            if not Merge_Option then
               Analyzer.Produce_Result (Unit_Name, Body_Part);
            end if;
         end if;
      end;
      Units_List.Skip;
   end loop;

   if Merge_Option then
      Analyzer.Produce_Result;
   end if;

   --
   -- Clean up
   --
   Asis.Ada_Environments.Close (My_Context);
   Asis.Ada_Environments.Dissociate (My_Context);
   Asis.Implementation.Finalize;
exception
   when Cancellation =>
      null;

   when Occur : Units_List.Specification_Error =>
      User_Message (To_Wide_String (Ada.Exceptions.Exception_Message (Occur)));
      Print_Help;

   when Asis.Exceptions.ASIS_Failed =>
      case Status is
         when Asis.Errors.Use_Error =>
            User_Message ("Inconsistent tree, please remove *.adt files");
         when others =>
            User_Message ("ASIS failure");
            Asis_Exception_Messages;
            raise;
      end case;

   when Occur :
     ASIS_Inappropriate_Context          |
     ASIS_Inappropriate_Container        |
     ASIS_Inappropriate_Compilation_Unit |
     ASIS_Inappropriate_Element          |
     ASIS_Inappropriate_Line             |
     ASIS_Inappropriate_Line_Number      =>
      User_Message ("ASIS error: " & To_Wide_String (Ada.Exceptions.Exception_Message (Occur)));
      Asis_Exception_Messages;
      raise;

   when Occur : others =>
      declare
         use Ada.Exceptions;
      begin
         User_Message ("Internal error: " & To_Wide_String (Exception_Name (Occur)));
         User_Message ("       Message: " & To_Wide_String (Exception_Message (Occur)));
         Stack_Traceback (Occur);
      end;
end Adadep;
